// import the packages like select,csv,scaleLinear,extent,axisLeft,axisBottom from the d3 library
import { 
  select, 
  csv, 
  scaleLinear, 
  extent,  
  axisLeft,
  axisBottom
} from 'd3';

// selection represents the SVG container where the scatter plot will be rendered, and props is an object containing various properties required for the scatter plot.

export const scatterPlot = (selection, props) => {
	const {
    xValue,
  	xAxisLabel,
  	yValue,
  	yAxisLabel,
    circleRadius,
  	margin,
  	width,
  	height,
  	data} = props;
// The function creates xScale and yScale using scaleLinear, which map the domain of data values to the range of the chart's inner width and height, respectively. 		
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
  	.domain(extent(data, xValue))
  	.range([0, innerWidth])
  	.nice();
  
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight, 0])
  	.nice();
 
 // The function creates xAxis and yAxis using axisBottom and axisLeft, respectively 
  const yAxis = axisLeft(yScale)
  							.tickSize(-innerWidth)
  							.tickPadding(5);
  
  const xAxis = axisBottom(xScale)
  								.tickSize(-innerHeight)
  								.tickPadding(20);
 
 // The function selects or appends a group element (g) with the class "container" and uses this group element to organize the elements of the scatter plot. 
  const g = selection.selectAll('.container').data([null]);
  
  const gEnter = g.enter().append('g').attr('class', 'container');
  
  gEnter.merge(g)
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
 // The function adds the x-axis and y-axis labels to the chart using the provided axis label texts and styles. 
	const yAxisG = g.select('.y-axis');
  const yAxisGenter = gEnter.append('g')
  			.attr('class', 'y-axis');
  yAxisG
  		.merge(yAxisGenter)
  			.call(yAxis)
  			.selectAll('.domain').remove();
  
  const yAxisLabelText = yAxisGenter.append('text')
  	.attr('class', 'axis-label')
  	.attr('fill', 'purple')
  	.attr('y', -93)
  	.attr('text-anchor', 'middle')
  	.attr('transform', `rotate(-90)`)
  .merge(yAxisG.select('.axis-label'))	
  	.attr('x', -innerHeight/2)
  	.text(yAxisLabel);
  
  
  const xAxisG = g.select('.x-axis');
  const xAxisGenter = gEnter.append('g')
  			.attr('class', 'x-axis');
  xAxisG
  		.merge(xAxisGenter)
  			.attr('transform', `translate(0, ${innerHeight})`)
  			.call(xAxis)
  			.selectAll('.domain').remove();
  
  const xAxisLabelText = xAxisGenter.append('text')
  	.attr('class', 'axis-label')
  	.attr('fill', 'Red')
  	.attr('y', 60)
  	.attr('text-anchor', 'middle')
  .merge(xAxisG.select('.axis-label'))	
  	.attr('x', innerWidth/2)
  	.text(xAxisLabel);
// The function selects or appends circle elements to represent the data points on the scatter plot.   
	const circles = g.merge(gEnter).selectAll('circle').data(data);
  circles.enter().append('circle')
    	.attr('cx', innerWidth / 2)
  		.attr('cy', innerHeight / 2)
  		.attr('r', 0)
    .merge(circles)
  		.transition().duration(1000)
    	.delay((d,i) => i * 10)
  		.attr('cy', d => yScale(yValue(d)))
  		.attr('cx', d => xScale(xValue(d)))
  		.attr('r', circleRadius);
  
}